import { invoke } from "@tauri-apps/api/tauri";
import { Subject, debounceTime, from } from "rxjs";

export interface PlanInterface {
    plan: PlanElement[];
    current: number;
    stored_path?: string;
    update_url?: string;
    name?: string;
}

export interface PlanMetadata {
    stored_path?: string;
    update_url?: string;
    name: string;
}

export class Plan {
    plan: PlanElement[];
    current: number;
    update_url?: string;
    name?: string;
    
    private path?: string;
    private selfSaveSubject: Subject<void> = new Subject<void>();

    constructor(plan: PlanInterface) {
        this.plan = plan.plan;
        this.current = plan.current;
        if (plan.stored_path) {
            this.path = plan.stored_path;
        }
        this.selfSaveSubject.pipe(debounceTime(500)).subscribe(() => this.directSelfSave());
    }

    setPath(path: string) {
        this.path = path;
    }

    next() {
        if (this.current + 1 < this.plan!.length) {
            this.current++;
            this.requestSelfSave();
        }
    }

    prev() {
        if (this.current - 1 >= 0) {
            this.current--;
            this.requestSelfSave();
        }
    }

    toInterface(): PlanInterface {
        return {
            plan: this.plan,
            current: this.current,
            stored_path: this.path,
            update_url: this.update_url,
        };
    }

    private requestSelfSave() {
        if (this.path) {
            this.selfSaveSubject.next();
        }
    }

    private directSelfSave() {
        invoke('save_plan_at_path', {path: this.path, plan: this.toInterface()});
    }

}

export interface PlanElement {
    area_key: string;
    notes?: string;
    uuid?: string;
    edited: boolean;
    anchor_act?: number;
}